/* Low-Cost Modulated Noise Interference Emulator Firmware v1.0 for Arduino Nano
 *   
 *  -----------------------------------------------------------------------------------------
 *  Author: Christian Spindelberger, TU Wien, EMCE, Gusshausstrasse 25/354,1040 Wien, Austria
 *  Revision: 1.0    Date: 2019/10/10
 *  This work is licensed under the Creative Commons Attribution-
 *  ShareAlike 4.0 International License. To view a copy of this license, 
 *  visit http://creativecommons.org/licenses/by-sa/4.0/ or send a letter
 *  to Creative Commons, PO Box 1866, Mountain View, CA 94042, USA.
 *  -----------------------------------------------------------------------------------------
 *  
 *  This code modulates noise bursts in terms of on-/off-times and power levels.
 *  The attenuators are not driven in latched mode! This means that the bitmask is set immediately. 
 *  Please use the additional Matlab class for interfacing the interference emulator. 
 *  
 *  Adaptation of the sequence length:
 *  The sequence length can be set in the code to the desired value by variable array_length. 
 *  Be careful, too high values may cause instable behavior of the microcontroller!
 *  
 *  Communication interface: 
 *  The noise source communicates over a serial interface (USB) and waits for a '\n' for termination.
 *    Start emulation -> Setting attenuation sequences corresponding to values in regB..D
 *    Stop emulation -> Stops the running output by setting maximum attenuation
 *    Data transfer -> Calls the function stream_data() for overwriting arrays regB..D with desired values 
 *    Print value 0..3*array_length-1 -> Returns the actual array value of the desired register
 *  
 *  Pin description:
 *  DDRD = B11111100; Digital pins D0-7
 *  DDRB = B00111111; Digital pins D8-13
 *  DDRC = B00111111; Analog pins A0-5 (4&5 are LE!!)
 *  
 *  ------------
 *  Attenuators: 
 *  ------------
 *  Attenuation stepped in for HIGH voltage!
 *  Mapping: 0.25dB-0.5dB-1dB-2dB-4dB-8dB-16dB
 *  D2-D8 = 0.25dB - 16dB @Att1
 *  A5 = LE @Att1
 *  D9-D12, A1-A0, D13 = 0.25dB - 16dB @Att2 
 *  A4 = LE @Att2
 */

// Variables
const int array_length = 200;
unsigned int on_times[array_length] = {0};
unsigned int off_times[array_length] = {0};
uint8_t regB[array_length] = {0};
uint8_t regC[array_length] = {0};
uint8_t regD[array_length] = {0};
char incomingBytes[20] = {0};
char data[20] = {0};
int start_emulation = 0;

// Functions
void communicate();
void emulate();
void flushBuffer();
void flushStreamBuffer();
void flushWordBuffer();
void stream_data();

// Initialization sequence
void setup() {
  Serial.begin(115200);
  DDRD = B11111100;
  DDRB = B00111111;
  DDRC = B00111111;
  PORTC = B00110000;
  PORTB = B00111111;
  PORTC = B00110011;
  PORTD = B11111100;
  PORTC = B00000011;
  }

// Main loop
void loop() {
  if (Serial.available() > 0){
    communicate();
    }
  if (start_emulation == 1) {
    emulate(); 
    }
  }

void emulate() {
  for (int i = 0; i < array_length; i++){ 
    PORTD = B11111100;
    PORTC = B00110011;
    PORTB = B00111111;
    delayMicroseconds(off_times[i]);   
    PORTD = byte(regD[i]);
    PORTC = byte(regC[i]);
    PORTB = byte(regB[i]);
    delayMicroseconds(on_times[i]);   
    }
  }

void flushBuffer() {
  while (Serial.read() >= 0);
  }

void flushStreamBuffer() {
  for (int i=0; i<20; i++){
    data[i] = '\0';
    }
  }
void stream_data() {
  int i = 0;
  for (int j=0; j<5; j++){
    while (1) {
        if (Serial.available()>0) {
          Serial.readBytesUntil('\n', data, 20);
          if (i == array_length){
            i = 0;
            break;
            }
          switch(j){
            case 0: regB[i] = atoi(data); break;
            case 1: regC[i] = atoi(data); break;
            case 2: regD[i] = atoi(data); break;
            case 3: on_times[i] = atoi(data); break;
            case 4: off_times[i] = atoi(data); break;
            } 
          flushBuffer();
          flushStreamBuffer();
          Serial.print("ACK\r\n");
          Serial.flush();
          i = i+1;
          }else if (i == array_length) {
          i = 0;
          break;
          }
        }
     }
  }

void flushWordBuffer() {
  for (int i=0; i<20; i++){
    incomingBytes[i] = '\0';
    }
  }

void communicate() {
    Serial.readBytesUntil('\n', incomingBytes, 20);
    
    if (strncmp(incomingBytes, "Start emulation", 15)==0){
      Serial.print("ACK\r\n");
      Serial.flush();
      start_emulation = 1;
    }else if (strncmp(incomingBytes, "Stop emulation", 14)==0){
      Serial.print("ACK\r\n");
      Serial.flush();
      start_emulation = 0;
      PORTC = B00110000;
      PORTB = B00111111;
      PORTC = B00110011;
      PORTD = B11111100;
      PORTC = B00000011;
    }else if (strncmp(incomingBytes, "Data transfer", 13)==0){
      flushStreamBuffer();
      Serial.print("ACK\r\n");
      Serial.flush();
      flushBuffer();
      start_emulation = 0;
      PORTC = B00110000;
      stream_data();
    }else if (strncmp(incomingBytes, "Print value", 11)==0){
      char value[5] = {0};
      for (int k = 12; k <16; k++){
        value[k-12] = incomingBytes[k];
        }
      value[4] = '\0';
      int index = atoi(value);
      if (index < array_length){
        Serial.print(regB[index]);
      }else if (index >= array_length && index < 2*array_length){
        index = index-array_length;
        Serial.print(regC[index]);
      }else if (index >= 2*array_length && index <3*array_length){
        index = index-2*array_length;
        Serial.print(regD[index]);
      }else if (index >= 3*array_length && index <4*array_length){
        index = index-3*array_length;
        Serial.print(on_times[index]);
      }else if (index >= 4*array_length && index <5*array_length){
        index = index-4*array_length;
        Serial.print(off_times[index]);
      }else{
        Serial.print("Index exceeds dimension!");
      }
      Serial.print("\r\n");
      Serial.flush();
    }else {
      Serial.print("Error\r\n");
      Serial.flush();
      }
    flushBuffer();
    flushWordBuffer();
  }
