% Author: Christian Spindelberger, TU Wien, EMCE, Gusshausstrasse 25/354,
% 1040 Wien, Austria
% Revision: 1.0     Date: 2019/10/10
% This work is licensed under the Creative Commons Attribution-
% ShareAlike 4.0 International License. To view a copy of this license, 
% visit http://creativecommons.org/licenses/by-sa/4.0/ or send a letter
% to Creative Commons, PO Box 1866, Mountain View, CA 94042, USA.

classdef noise_source < handle 
   properties
       baudrate; port; uC;
       reply; reg_vals; rx_array; cmd;
   end
   
   methods
       function obj = noise_source()
            obj.baudrate = 115200;
            obj.port = seriallist;
            obj.uC = serial(obj.port);
            set(obj.uC,'BaudRate',obj.baudrate);
            fopen(obj.uC);
            pause(5);
       end
       
       function val = tx_cmd(obj)
            flushinput(obj.uC);
            fprintf(obj.uC,obj.cmd);
            obj.cmd = [];
            obj.reply = fscanf(obj.uC);
            if strncmp(obj.reply, "ACK", 3)
                val = 1;
            else
                val = 0;
            end
       end
       
       function val = start_emulation(obj)
            obj.cmd = "Start emulation\n";
            val = obj.tx_cmd();
            if val
                disp("Emulation started.");
            else
                disp("Error: Could not start emulation.");
            end
       end
       
       function val = stop_emulation(obj)
           obj.cmd = "Stop emulation\n";
           val = obj.tx_cmd();
           if val
                disp("Emulation stopped.");
           else
                disp("Error: Could not start emulation.");
           end
       end
       
       function val = data_transfer(obj)
            obj.cmd = "Data transfer\n";
            val = obj.tx_cmd();
            if val
                disp("Ready for data!");
            else
                disp("Error: Please try again!");
            end
       end
       
       function val = check_sum(obj)
           obj.rx_array = zeros(1, numel(obj.reg_vals));
           s3 = "\n";
           for i = 0:(numel(obj.reg_vals)-1)
                s1 = "Print value ";
                if i<10
                    s2 = sprintf("000%d", i);
                elseif i>=10 && i<100
                    s2 = sprintf("00%d", i);
                elseif i>=100 && i<1000
                    s2 = sprintf("0%d", i);
                else 
                    s2 = sprintf("%d", i);
                end
                obj.cmd = strcat(s1,s2,s3);
                obj.tx_cmd();
                obj.rx_array(i+1) = str2num(obj.reply);
           end

           if isequal(obj.rx_array, obj.reg_vals)
               val = 1;
           else
               val = 0;
           end
       end
        
       function val = stream_data(obj)
            disp("Streaming data...");
            tx_counter = 0;
            error_counter = 0;
            for i = 1:numel(obj.reg_vals)
                flushinput(obj.uC);
                fprintf(obj.uC,"%u\n",obj.reg_vals(i));
                obj.reply = fscanf(obj.uC);
                if strncmp(obj.reply, "ACK", 3)
                    tx_counter = tx_counter+1;
                else
                    error_counter = error_counter+1;
                    break;
                end
            end
    
            if error_counter > 0
                disp("Error: Reset Controller!");
                val = 0;
            elseif tx_counter == numel(obj.reg_vals)
                if obj.check_sum()
                    disp("Data transfer completed.");
                    val = 1;
                else
                    disp("Error: Reset Controller!");
                    val = 0;
                end
            else
                disp("Error: Reset Controller!");
                val = 0;
            end
       end
       
       function amp_encoding(obj, N_vals, varargin)
            p = inputParser;
            default_array = [];
            validPDF = @(x) (isprop(x, 'DistributionName'));
            addRequired(p, 'N_vals', @(x) (x > 0));
            addParameter(p, 'BW', @(x) (x > 0));
            addParameter(p, 'pdf', validPDF);
            addParameter(p, 'custom_amps', default_array, @isrow);
            parse(p, N_vals, varargin{:});
            
            if isempty(p.Results.custom_amps)
                noise_floor = -174+10*log10(p.Results.BW);
                amps = random(p.Results.pdf, 1, N_vals);
                att = amps-noise_floor;
            else
                att = p.Results.custom_amps;
            end
            
            partition = 0.125:0.25:63.375;
            codebook = 0:0.25:63.5;
            [~, att_q] = quantiz(att, partition, codebook);
            
            if isempty(p.Results.custom_amps)
                att_q = abs(att_q-63.5);
            end

            regD = zeros(1,N_vals);
            regB = zeros(1,N_vals);
            regC = zeros(1,N_vals);

            for j = 1:N_vals
                if (att_q(j) <= 31.75 && att_q(j) >= 0)
                    reg = [flip(bitget((2^7-1)*(att_q(j))/31.75,1:7)),...
                        flip(bitget(0,1:7))];
                elseif (att_q(j) > 31.75 && att_q(j) <= 63.5)
                    reg = [flip(bitget(2^7-1,1:7)),...
                        flip(bitget((2^7-1)*(att_q(j)-31.75)/31.75,1:7))];   
                else
                    disp("Error");
                end
                
                a1_16=reg(1); a1_8=reg(2); a1_4=reg(3); a1_2=reg(4);...
                    a1_1=reg(5); a1_05=reg(6); a1_025=reg(7);
                a2_16=reg(8); a2_8=reg(9); a2_4=reg(10); a2_2=reg(11);...
                    a2_1=reg(12); a2_05=reg(13); a2_025=reg(14);
                valD = [a1_8, a1_4, a1_2, a1_1, a1_05, a1_025,...
                    zeros(1,2)];
                valB = [zeros(1,2), a2_16, a2_2, a2_1, a2_05, a2_025,...
                    a1_16];
                valC = [zeros(1,2), ones(1,2), zeros(1,2), a2_4, a2_8];
   
                regD(j) = bin2dec(sprintf("%d",valD));
                regB(j) = bin2dec(sprintf("%d",valB));
                regC(j) = bin2dec(sprintf("%d",valC));
            end
            
            obj.reg_vals = [regB, regC, regD];
       end
       
       function stream_custom_data(obj, emulation_mode, att_vals, varargin)
           expectedModes = {'time quantized noise', 'modulated noise'};
           defaultArray = [];
           p = inputParser;
           validMode = @(x) any(validatestring(x,expectedModes));
           addRequired(p, 'emulation_mode', validMode);
           addRequired(p, 'att_vals', @isrow);
           addParameter(p, 'On_times', defaultArray, @isrow);
           addParameter(p, 'Off_times', defaultArray, @isrow);
           parse(p, emulation_mode, att_vals, varargin{:});
           
           obj.stop_emulation();
           obj.amp_encoding(numel(p.Results.att_vals), 'custom_amps',...
               p.Results.att_vals);
           
           if strcmp(p.Results.emulation_mode, 'modulated noise') 
                obj.reg_vals = [obj.reg_vals, p.Results.On_times,...
                    p.Results.Off_times];
           end
           
           while (1)
               if obj.data_transfer()
                   if obj.stream_data()
                       break;
                   end
               end
           end
           obj.start_emulation();
       end
       
       function stream_sequence(obj, emulation_mode, array_length,...
               pdf_power, varargin)
           expectedModes = {'time quantized noise', 'modulated noise'};
           defaultBW = 20e6;
           
           p = inputParser;
           validMode = @(x) any(validatestring(x,expectedModes));
           validPDF = @(x) (isprop(x, 'DistributionName'));
           addRequired(p, 'emulation_mode', validMode);
           addRequired(p, 'array_length', @(x) (x > 0));
           addRequired(p, 'pdf_power', validPDF);
           addOptional(p, 'bandwidth', defaultBW, @(x) (x > 0));
           addParameter(p, 'PDF_on', validPDF);
           addParameter(p, 'PDF_off', validPDF);
           parse(p, emulation_mode, array_length, pdf_power, varargin{:});
           
           obj.stop_emulation();
           obj.amp_encoding(p.Results.array_length, 'BW',...
               p.Results.bandwidth, 'pdf', p.Results.pdf_power);
           
           if strcmp(p.Results.emulation_mode, 'modulated noise') 
                on_times = round(random(p.Results.PDF_on, 1,...
                    p.Results.array_length));
                off_times = round(random(p.Results.PDF_off, 1,...
                    p.Results.array_length));
                obj.reg_vals = [obj.reg_vals, on_times, off_times];
           end
           
           while (1)
               if obj.data_transfer()
                   if obj.stream_data()
                       break;
                   end
               end
           end
           obj.start_emulation();
        end
        
       function delete(obj)
          fclose(obj.uC); 
       end
   end
end