% Author: Christian Spindelberger, TU Wien, EMCE, Gusshausstrasse 25/354,
% 1040 Wien, Austria
% Revision: 1.0     Date: 2019/10/10
% This work is licensed under the Creative Commons Attribution-
% ShareAlike 4.0 International License. To view a copy of this license, 
% visit http://creativecommons.org/licenses/by-sa/4.0/ or send a letter
% to Creative Commons, PO Box 1866, Mountain View, CA 94042, USA.

%% Low-Cost Interference Emulator Example Code
% In this example the different working modes and several commands
% are presented. Please make sure that the right Arduino firmware version 
% was flashed to the microcontroller before! In the example code three
% distribution files are available to generate random on-/off-times, and
% power level arrays. For further details about the working principle 
% itself, schematics, firmware considerations and Matlab functions please 
% refer to the report file on http://www.interreg-interop.eu/fileadmin/...
% t/InterOp/low_cost_interference_emulator.pdf

%% Setup Instructions
% Ensure that the diode between USB connector power supply and
% microcontroller is desolderd on the Arduino Nano board. Adjust the
% primary power supply for the microcontroller, power amplifier, etc. to
% 4.2V (current limit 500mA) and the diode biasing to 13V (current limit
% 100mA). First, turn on the primary supply and proceed then with the
% biasing, to ensure a stable behavior of the power amplifiers.
% Connect the USB cable to the PC which is running Matlab. Flash
% the desired firmware (time-quantized method or modulated noise) to the
% controller and proceed with the Matlab script. Depending on the used
% method un-/comment the code below.

% Clearing variaibles, workspace and deleting communication ports.
clc, clear, close all;
instrreset;

% Creating an object of the noise source.
dev = noise_source();

%% Time-quantized method
% Power levels generated by random variable. Adjust the array_length
% corresponding to the array_length in the firmware!
array_length = 500;
% Loading random variable of power level statistics. 
load('Power_level_dist.mat'); % Power levels (dBm)
% Setting the equivalent noise power bandwidth as reference power level
% relative to the power levels from the statistics to 22MHz.
ref_np_bw = 22e6;
% Streams 500 power level samples to the noise source and starts the
% emulation with a time quantization (firmware: variable set_delay) of 1ms.
dev.stream_sequence('time quantized noise', array_length, pd_power, ...
    'bandwidth', ref_np_bw);

% Custom attenuation values can also be streamed. Individual attenuation
% states att_states are created and streamed to the source using the
% following function.
% att_states = 0*ones(1,array_length);
% dev.stream_custom_data('time quantized noise', att_states);

%% Modulated noise method
% Parameters generated by random variables. Adjust the array_length
% corresponding to the array_length in the firmware!
array_length = 200;
% Loading random variables.
load('On_times_dist.mat');
load('Off_times_dist.mat'); % On-/off-times (s)
load('Power_level_dist.mat'); % Power levels (dBm)
% Setting the equivalent noise power bandwidth as reference power level
% relative to the power levels from the statistics to 22MHz.
ref_np_bw = 22e6;
dev.stream_sequence('modulated noise', array_length, pd_power, 'PDF_on', ...
     pd_on, 'PDF_off', pd_off, 'bandwidth', ref_np_bw);
 
% Custom parameters
% att_states = 0*ones(1,array_length);
% on_times = ones(1,array_length);
% off_times = ones(1,array_length);
% dev.stream_custom_data('modulated noise', att_states, 'On_times', ...
%      on_times, 'Off_times', off_times);

%% Commands
% Starts emulation corresponding to timing and attenuation parameters.
dev.start_emulation();

% Stops emulation and sets attenuators to maximum attenuation value of
% 63.5dB.
dev.stop_emulation();

% Deleting the object.
dev.delete();
